﻿using GiftShopContracts.BindingModels;
using GiftShopContracts.SearchModels;
using GiftShopContracts.StoragesContracts;
using GiftShopContracts.ViewModels;
using GiftShopDatabaseImplement.Models;

namespace GiftShopDatabaseImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {

        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new GiftShopDatabase();
            var element = context.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (element == null)
            {
                return null;
            }
            context.Orders.Remove(element);
            context.SaveChanges();
            return AcessGiftsStorage(element.GetViewModel, context);
        }


        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            using var context = new GiftShopDatabase();
            return AcessGiftsStorage(context.Orders.FirstOrDefault(x => x.Id == model.Id)?.GetViewModel, context);
        }

        public List<OrderViewModel?> GetFilteredList(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return new();
            }
            using var context = new GiftShopDatabase();
            return context.Orders.Where(x => x.Id == model.Id).Select(x => AcessGiftsStorage(x.GetViewModel, context)).ToList();
        }

        public List<OrderViewModel?> GetFullList()
        {
            using var context = new GiftShopDatabase();
            return context.Orders.Select(x => AcessGiftsStorage(x.GetViewModel, context)).ToList();
        }


        public OrderViewModel? Insert(OrderBindingModel model)
        {
            var order = Order.Create(model);
            if (order == null)
            {
                return null;
            }
            using var context = new GiftShopDatabase();
            context.Orders.Add(order);
            context.SaveChanges();
            return AcessGiftsStorage(order.GetViewModel, context);

        }


        public OrderViewModel? Update(OrderBindingModel model)
        {

            using var context = new GiftShopDatabase();
            var order = context.Orders.FirstOrDefault(x => x.Id == model.Id);
            if (order == null)
            {
                return null;
            }
            order.Update(model);
            context.SaveChanges();
            return AcessGiftsStorage(order.GetViewModel, context);

        }

        public static OrderViewModel? AcessGiftsStorage(OrderViewModel? model, GiftShopDatabase context)
        {
            if (model == null)
            {
                return null;
            }
            var gifts = context.Gifts.FirstOrDefault(x => x.Id == model.GiftId);
            if (gifts == null)
            {
                return null;
            }
            model.GiftName = gifts.GiftName;
            return model;
        }

    }
}
